﻿using UnityEngine;
using UnityEngine.UI;
using ScrambleKit.FX;
using DG.Tweening;

public class Combo : MonoBehaviour
{
    public static int TotalScore;
    public static int BestCombo;

    public static void RegisterKill()
    {
        instance.ScoreKill();
        instance.CelebrateKill();
    }
    private static Combo instance;


    public Text scoreText;
    public Text comboCard;
    public Text killCard;

    public float comboTimeLimit = 1f;

    private float lastKillTime;
    private int comboCount;

    private Vector3 comboCardStartPos;
    private Vector3 killCardStartPos;

    void Start()
    {
        instance = this;
        TotalScore = 0;
        BestCombo = 0;

        comboCardStartPos = comboCard.transform.position;
        killCardStartPos = killCard.transform.position;
    }

    private void ScoreKill()
    {
        if(Time.fixedTime - lastKillTime <= comboTimeLimit)
        {
            comboCount++;
            if(comboCount>2)
                FireComboCard(comboCount);
            if (comboCount > BestCombo)
                BestCombo = comboCount;
        }
        else
        {
            comboCount = 1;
        }

        int killScore = 50 * comboCount;
        TotalScore += killScore;
        scoreText.text = TotalScore.ToString("#,#");

        FireKillCard(killScore);

        lastKillTime = Time.fixedTime;
    }

    private void FireComboCard(int combo)
    {
        DOTween.Kill(comboCard.transform);
        ClearComboCard();

        comboCard.text = combo + "x" + ((combo>=5) ? "!":"");
        comboCard.transform.DOPunchScale(Mathf.Min(3.5f,0.5f * combo) * Vector3.one, 0.5f);
        comboCard.transform.DOPunchRotation(new Vector3(0,0,30), 0.8f, 2).OnComplete(ClearComboCard);
    }

    private void ClearComboCard()
    {
        comboCard.text = "";
        comboCard.transform.position = comboCardStartPos;
        comboCard.transform.localEulerAngles = Vector3.zero;
        comboCard.transform.localScale = Vector3.one;
    }

    private void FireKillCard(int killscore)
    {
        DOTween.Kill(killCard.transform);
        ClearKillCard();

        killCard.text = "+" + killscore;        
        killCard.transform.DOMove(killCardStartPos + new Vector3(0, 300, 0), 1f).SetEase(Ease.InOutSine).OnComplete(ClearKillCard);
    }

    private void ClearKillCard()
    {
        killCard.text = "";
        killCard.transform.position = killCardStartPos;
    }

    private void CelebrateKill()
    {
        if (comboCount >= 5)
            SoundFX.PlaySound("Combo5");
        else if (comboCount == 4)
            SoundFX.PlaySound("Combo4");
        else if (comboCount == 3)
            SoundFX.PlaySound("Combo3");
        else if (comboCount == 2)
            SoundFX.PlaySound("Combo2");
        else
            SoundFX.PlaySound("Combo1");
    }

}
