﻿using DG.Tweening;
using UnityEngine;


public class HopperMove : MonoBehaviour
{
    public string DEBUG;

    public Transform foot;
    public Transform head;

    private Rigidbody rb;

    void Start()
    {
        rb = GetComponent<Rigidbody>();
        ScheduleAction();
    }

    void ScheduleAction()
    {
        rb.StopInstantly();
        rb.isKinematic = true;
        Invoke("Act", 1);
    }

    void Act()
    {
        RaycastHit hitInfo;
        
        bool anyHit = Physics.Raycast(transform.position + Vector3.up, transform.forward, out hitInfo, 3.9f);

        //Debug.DrawLine(transform.position + Vector3.up, transform.position + 6f * transform.forward + Vector3.up, Color.white, 2f);

        if(!anyHit)
        {
            DEBUG = "no hit";
            Jump();
        }
        else if(hitInfo.transform.GetComponent<Mother>() != null)
        {
            DEBUG = "mother";
            Attack();
        }
        else
        {
            DEBUG = hitInfo.transform.name + " = idle";
            IdleHop();
        }
    }

    private void IdleHop()
    {
        rb.isKinematic = false;
        Sequence footAnim = DOTween.Sequence();
        footAnim.Append(foot.DOLocalMoveY(.825f, 0.2f).SetEase(Ease.OutCubic));
        footAnim.Append(foot.DOLocalMoveY(1, 0.4f).SetEase(Ease.InOutSine));

        rb.DOJump(transform.position + new Vector3(0, 0, 0), 3, 1, 0.6f, false).OnComplete(ScheduleAction);
    }

    private void Jump()
    {
        rb.isKinematic = false;
        Sequence footAnim = DOTween.Sequence();
        footAnim.Append(foot.DOLocalRotate(new Vector3(45, 0, 0), 0.25f).SetEase(Ease.InCubic));
        footAnim.Append(foot.DOLocalRotate(new Vector3(-30, 0, 0), 0.15f));
        footAnim.Append(foot.DOLocalRotate(new Vector3(0, 0, 0), 0.2f).SetEase(Ease.InOutSine));

        rb.DOJump(transform.position + new Vector3(0, 0, -3), 2, 1, 0.6f, false).OnComplete(ScheduleAction);        
    }

    private void Attack()
    {
        Sequence attackAnim = DOTween.Sequence();
        attackAnim.Append(head.DOLocalRotate(new Vector3(-45f, 0, 0), 0.3f));
        attackAnim.Append(head.DOLocalRotate(new Vector3(90f, 0, 0), 0.25f).SetEase(Ease.InCubic).OnComplete(DealDamage));
        attackAnim.Append(head.DOLocalRotate(new Vector3(0, 0, 0), 0.15f).OnComplete(ScheduleAction));
    }

    private void DealDamage()
    {
        Mother.instance.TakeDamage();
    }
}
