﻿using UnityEngine;
using System.Collections.Generic;
using System.Linq;
using DG.Tweening;

public class Lance : MonoBehaviour
{
    public List<Transform> fists;
    private Dictionary<Transform, Vector3> restPos;

    void Start()
    {
        restPos = new Dictionary<Transform, Vector3>();
        foreach(Transform fist in fists)
        {
            restPos[fist] = fist.localPosition;
        }
    }

    void OnTriggerEnter(Collider c)
    {
        if (c != null && c.transform != null && c.transform.tag == "Enemy")
        {
            c.transform.BroadcastMessage("OnLancing", SendMessageOptions.DontRequireReceiver);
            Punch(c.transform.position);
            Invoke("VictoryCheck", 1.5f);
        }
    }

    void VictoryCheck()
    {
        WaveLauncher.CheckForVictory();
    }

    void Punch(Vector3 position)
    {
        Transform fist = GetClosestFist(position);
        if (fist == null)
            return;

        Vector3 height = (1 + Random.Range(0f, 2f)) * Vector3.up;
        Vector3 localPunchTarget = 0.7f * fist.parent.InverseTransformPoint(position + height);
        float rotSign = Mathf.Sign(restPos[fist].x); // we're late-jammin now!  left vs right fist rotate opposite 

        DOTween.Kill(fist);
        Sequence punch = DOTween.Sequence();
        punch.Append(fist.DOLocalMove(localPunchTarget, 0.1f).SetEase(Ease.OutCubic));
        punch.Append(fist.DOLocalMove(restPos[fist], 0.05f).SetEase(Ease.InOutSine));
        punch.Insert(0, fist.DOLocalRotate(new Vector3(0, 0, rotSign * 50f), 0.04f));
        punch.Insert(0.1f, fist.DOLocalRotate(new Vector3(0, 0, 0), 0.05f));
    }

    private Transform GetClosestFist(Vector3 position)
    {
        return fists.OrderBy(f => (f.position - position).magnitude).FirstOrDefault();
    }
}
