﻿using System;
using System.Collections.Generic;
using UnityEngine;
using ScrambleKit.FX;
using UnityEngine.SceneManagement;

public class WaveLauncher : MonoBehaviour
{
    [Serializable]    
    public class Wave
    {
        public Transform wavePrefab;
        public float launchTime;
        [HideInInspector]
        public bool launched;
    }

    private static WaveLauncher instance;
    public static void CheckForVictory()
    {
        GameObject[] enemies = GameObject.FindGameObjectsWithTag("Enemy");
        if (enemies.Length > 0)
            return;

        if (instance.AnyWavesRemain())
            return;

        SceneManager.LoadScene("VictoryScene");
    }

    public List<Wave> waves;
    public FXTrigger launchFX;

    private float startTime;
    private float delay;

    void Start()
    {
        instance = this;
        delay = 0;
        startTime = Time.fixedTime;
    }

    void FixedUpdate()
    {
        foreach(Wave wave in waves)
        {
            if(!wave.launched && Elapsed() >= wave.launchTime)
            {
                LaunchWave(wave);
            }
        }
    }

    private bool AnyWavesRemain()
    {
        foreach (Wave wave in waves)
        {
            if (!wave.launched)
                return true;
        }
        return false;
    }

    private void LaunchWave(Wave wave)
    {
        wave.launched = true;
        Transform t = Instantiate(wave.wavePrefab);
        t.position = transform.position;
        launchFX.Trigger(transform);
    }

    private float Elapsed()
    {
        return Time.fixedTime - (startTime + delay);
    }
}
