﻿using UnityEngine;

public class Lean : MonoBehaviour
{
    public Transform body;

    public string DEBUG;
    public Transform head;
    public Transform face;

    public float leanFactor = 3;
    public float leanMax = 20;
    public float headCounterLeanFactor = 0.9f;
    public float headRaiseFactor = 0.9f;

    private PlayerMovementTr move;
    private MouseAim aim;

    void Start()
    {
        move = GetComponent<PlayerMovementTr>();
        aim = GetComponent<MouseAim>();
    }

    void Update()
    {
        float diff = Mathf.DeltaAngle(head.localEulerAngles.y, 0);
        float sideLean = Mathf.Clamp(diff * leanFactor, -leanMax, leanMax);

        float bodyLeanTween = Mathf.LerpAngle(transform.localEulerAngles.z, sideLean, 0.3f);
        transform.localEulerAngles = new Vector3(transform.localEulerAngles.x, transform.localEulerAngles.y, bodyLeanTween);

        float headLeanTween = Mathf.LerpAngle(head.localEulerAngles.z, -sideLean * headCounterLeanFactor, 0.2f);
        head.localEulerAngles = new Vector3(head.localEulerAngles.x, head.localEulerAngles.y, headLeanTween);

        aim.leanOffset = Mathf.Abs(sideLean * headRaiseFactor);

        face.localEulerAngles = new Vector3(
            Mathf.LerpAngle(0f, head.localEulerAngles.x, 0.2f),
            Mathf.LerpAngle(0f, head.localEulerAngles.y, 0.1f),
            Mathf.LerpAngle(0f, -transform.localEulerAngles.z, 0.7f)
            );

        float forwardLean = Mathf.DeltaAngle(head.localEulerAngles.x, 0);
        face.localPosition = new Vector3(
                face.localPosition.x,
                face.localPosition.y,
                Mathf.Lerp(face.localPosition.z, forwardLean * 0.0075f, 0.5f)
            );
    }

    void GarbageFailUpdate()
    {
        float xaccel = move.perceivedHorizAccel.x;
        float sideLean = Mathf.Clamp(-xaccel * leanFactor, -leanMax, leanMax);

        float bodyLeanTween = transform.localEulerAngles.z * .95f + 0.05f * sideLean;
        transform.localEulerAngles = new Vector3(transform.localEulerAngles.x, transform.localEulerAngles.y, bodyLeanTween);

        float headLeanTween = head.localEulerAngles.z * .95f + 0.05f * (-sideLean * headCounterLeanFactor);
        head.localEulerAngles = new Vector3(head.localEulerAngles.x, head.localEulerAngles.y, headLeanTween);

        DEBUG = Mathf.RoundToInt(sideLean) + ", " + bodyLeanTween;
    }
}
