﻿using UnityEngine;

public class PlayerMovementTr : MonoBehaviour
{
    public string DEBUG = "";

    public float maxWalkSpeed = 12f;
    public float walkAccel = 36f;
    public float airWalkAccel = 6f;
    public float jumpLaunchSpeed = 3.5f;
    public float gravityFactor = 1f;

    public Transform directionDriver;
    public Transform leanDriver;

    public Vector2 walkInputDir;
    public Vector3 perceivedHorizAccel;

    public float jumpTriggerTime;
    public float lastJumpTime;
    public void TriggerJump()
    {
        jumpTriggerTime = Time.fixedTime;
    }

    private float lastGroundedTime;

    public Vector3 velocity;
    private Vector3 instantAccel;

    private CharacterController ctrl;
    void Start()
    {
        if (directionDriver == null)
            directionDriver = transform;
        if (leanDriver == null)
            leanDriver = transform;

        ctrl = GetComponent<CharacterController>();
        velocity = Vector3.zero;
        lastGroundedTime = -999f;
        jumpTriggerTime = -999f;
    }

    void FixedUpdate()
    {
        instantAccel = Vector3.zero;
        CheckGrounded();

        if (RecentlyGrounded())
        {
            GroundWalk();
            GroundJump();
        }
        else
        {
            AirWalk();
        }

        Gravity();

        perceivedHorizAccel = leanDriver.InverseTransformVector(instantAccel.ToHoriz()) / Time.fixedDeltaTime;
        velocity += instantAccel;
        ctrl.Move(velocity * Time.fixedDeltaTime);
        ReduceVelocityFromCollisions();
    }

    private void ReduceVelocityFromCollisions()
    {
        //velocity = ctrl.velocity;  // was not good enough because collision-correction parts of move would "pop" player
        // so the new rule is that ctrl can only REDUCE the velocity, eg from running into a wall or falling onto the floor
        // and that should never INCREASE velocity/momentum for any reason

        velocity = new Vector3(
            LowerAbsolute(velocity.x, ctrl.velocity.x),
            LowerAbsolute(velocity.y, ctrl.velocity.y),
            LowerAbsolute(velocity.z, ctrl.velocity.z)
        );

    }

    private float LowerAbsolute(float a, float b)
    {
        if (Mathf.Abs(a) <= Mathf.Abs(b))
            return a;
        else
            return b;
    }

    private void GroundWalk()
    {
        Vector3 worldMoveDir = directionDriver.TransformDirection(new Vector3(walkInputDir.x, 0, walkInputDir.y));
        Vector3 accel = transform.GetHorizOnlyAccelTowardVelocityByAtMost(velocity, worldMoveDir * maxWalkSpeed, walkAccel);
        instantAccel += accel;

        // DEBUG =  "a=" + accel + " w=" + worldMoveDir;
    }

    private void AirWalk()
    {
        Vector3 worldMoveDir = transform.TransformDirection(new Vector3(walkInputDir.x, 0, walkInputDir.y)).normalized;
        instantAccel += transform.GetHorizOnlyAccelTowardVelocityByAtMost(velocity, worldMoveDir * maxWalkSpeed, airWalkAccel);
    }

    private void GroundJump()
    {
        if (!RecentlyTriggered())
        {
            return;
        }

        if (JustJumped())
            return;

        // velocity = new Vector3(velocity.x, jumpLaunchSpeed, velocity.z);
        instantAccel += new Vector3(0, jumpLaunchSpeed, 0);

        jumpTriggerTime = -999;
        lastJumpTime = Time.fixedTime;
    }

    private void Gravity()
    {
        if (!CurrentlyGrounded())
            instantAccel += gravityFactor*Physics.gravity*Time.fixedDeltaTime;
    }

    private bool CurrentlyGrounded()
    {
        return lastGroundedTime >= Time.fixedTime - 0.01f;
    }

    private bool RecentlyGrounded()
    {
        return lastGroundedTime >= Time.fixedTime - 0.1f;
    }

    private bool JustJumped()
    {
        return lastJumpTime >= Time.fixedTime - 0.1f;
    }

    private bool RecentlyTriggered()
    {
        return jumpTriggerTime >= Time.fixedTime - 0.3f;
    }

    private void CheckGrounded()
    {
        if (ctrl.isGrounded)
        {
            lastGroundedTime = Time.fixedTime;
        }

        //RaycastHit hit;
        //Physics.Raycast(transform.position, Vector3.down, out hit, 1.1f);

        //if (hit.transform)
        //{
        //    lastGroundedTime = Time.fixedTime;
        //}
    }
}

