﻿using UnityEngine;

public class GunRaycast : MonoBehaviour
{
    public Transform marker;
    public float refireDelay = 0.3f;
    public int damage = 1;

    private float triggerPulledAt;
    private float lastFireTime;

    private GunProjectile projectile;
    void Start() { projectile = GetComponent<GunProjectile>(); }

    void Update()
    {
        if (Input.GetMouseButtonDown(0))
            triggerPulledAt = Time.fixedTime;

        DrawDebugRay();
        if (!RecentlyTriggered() || InRefireDelay())
            return;

        Shoot();
    }

    private void DrawDebugRay()
    {
        Transform origin = Camera.main.transform;
        Debug.DrawRay(origin.position, 100f*origin.forward, Color.white);

        RaycastHit hit;
        Physics.Raycast(origin.position, origin.forward, out hit, 200f);
        if (hit.transform)
        {
            if (marker != null)
                marker.position = hit.point;
        }
    }

    private void Shoot()
    {
        lastFireTime = Time.fixedTime;

        RaycastHit hit;
        Transform origin = Camera.main.transform;
        Physics.Raycast(origin.position, origin.forward, out hit, 200f);


        if (projectile != null)
            FireProjectile(hit);
        else
            FireRay(hit);
    }

    private void FireProjectile(RaycastHit hit)
    {
        Vector3 targetPos = Camera.main.transform.position + projectile.horizonDistance* Camera.main.transform.forward;
        if (hit.transform)
            targetPos = hit.point;

        projectile.FireAt(targetPos);
    }

    private void FireRay(RaycastHit hit)
    { 
        if(hit.transform)
        {
            hit.transform.BroadcastMessage("TakeDamage", damage, SendMessageOptions.DontRequireReceiver);
        }
        // TODO else misfire into the sky or whatever, but still trigger FX when it's here
    }


    private bool RecentlyTriggered()
    {
        return triggerPulledAt >= Time.fixedTime - 0.1f;
    }

    private bool InRefireDelay()
    {
        return lastFireTime >= Time.fixedTime - refireDelay;
    }

}
