﻿using UnityEngine;

public class PlayerMovement : MonoBehaviour
{
    public string DEBUG = "";

    public float maxWalkSpeed = 12f;
    public float walkAccel = 36f;
    public float airWalkAccel = 6f;
    public float jumpLaunchSpeed = 3.5f;

    public Vector2 walkInputDir;

    private float jumpTriggerTime;
    private float lastJumpTime;
    public void TriggerJump()
    {
        jumpTriggerTime = Time.fixedTime;
    }

    private float lastGroundedTime;

    private Rigidbody rb;
    void Start()
    {
        rb = GetComponent<Rigidbody>();
        lastGroundedTime = -999f;
        jumpTriggerTime = -999f;
    }

    void FixedUpdate()
    {
        CheckGrounded();

        if (RecentlyGrounded())
        {
            GroundWalk();
            GroundJump();
        }
        else
        {
            AirWalk();
        }
    }

    private void GroundWalk()
    {
        Vector3 worldMoveDir = transform.TransformDirection(new Vector3(walkInputDir.x, 0, walkInputDir.y));
        rb.HorizOnlyAccelTowardVelocityByAtMost(worldMoveDir * maxWalkSpeed, walkAccel);
    }

    private void AirWalk()
    {
        Vector3 worldMoveDir = transform.TransformDirection(new Vector3(walkInputDir.x, 0, walkInputDir.y));
        rb.HorizOnlyAccelTowardVelocityByAtMost(worldMoveDir * maxWalkSpeed, airWalkAccel);
    }

    private void GroundJump()
    {
        if (!RecentlyTriggered() || JustJumped())
        {
            return;
        }

        Vector3 accel = jumpLaunchSpeed * Vector3.up;
        Vector3 forceAccel = rb.mass * accel;
        rb.AddForce(forceAccel, ForceMode.Impulse);

        jumpTriggerTime = -999;
        lastJumpTime = Time.fixedTime;
    }

    private bool RecentlyGrounded()
    {
        return lastGroundedTime >= Time.fixedTime - 0.1f;
    }

    private bool JustJumped()
    {
        return lastJumpTime >= Time.fixedTime - 0.1f;
    }

    private bool RecentlyTriggered()
    {
        return jumpTriggerTime >= Time.fixedTime - 0.3f;
    }

    private void CheckGrounded()
    {
        RaycastHit hit;
        Physics.Raycast(transform.position, Vector3.down, out hit, 1.1f);

        if(hit.transform)
        {
            DEBUG = "grounded by " + hit.transform.name;
            lastGroundedTime = Time.fixedTime;
        }
        else
        {
            DEBUG = "ungrounded";
        }
    }
}
