﻿using UnityEngine;

public static class RidigbodyExtensions
{

    public static void HorizOnlyAccelTowardVelocityByAtMost(this Rigidbody rb, Vector3 targetVelocity, float maxAccelPerSec)
    {
        // strip y, which is vertical
        Vector3 rbHoriz = new Vector3(rb.velocity.x, 0, rb.velocity.z);
        Vector3 targetHoriz = new Vector3(targetVelocity.x, 0, targetVelocity.z);

        rb.Accel(rbHoriz, targetHoriz, maxAccelPerSec);
    }

    public static void AccelTowardVelocityByAtMost(this Rigidbody rb, Vector3 targetVelocity, float maxAccelPerSec)
    {
        rb.Accel(rb.velocity, targetVelocity, maxAccelPerSec);
    }

    private static void Accel(this Rigidbody rb, Vector3 currVelocity, Vector3 targetVelocity, float maxAccelPerSec)
    {
        Vector3 deltaVel = targetVelocity - currVelocity;
        float changePossible = Mathf.Min(deltaVel.magnitude / Time.fixedDeltaTime, maxAccelPerSec);

        Vector3 accel = changePossible * deltaVel.normalized;
        Vector3 forceAccel = rb.mass * accel;
        rb.AddForce(forceAccel);
    }

    public static void StopInstantly(this Rigidbody rb)
    {
        rb.AddForce(rb.mass * -rb.velocity / Time.fixedDeltaTime);
    }

}
